//-----------------------------------------------------------------------------
// File: BasicHLSL.fx
//
// Desc: The effect file for the BasicHLSL sample.  It implemetns 
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 g_MaterialAmbientColor;	// Material's ambient color
float4 g_MaterialDiffuseColor;	// Material's diffuse color

// This effect file uses a single directional light
float4 lhtDir = {0.0f, 0.0f, -1.0f, 1.0f};    //light Direction 
float3 g_LightDir = normalize(float3(1.0f, 1.0f, 1.0f));	// Light's direction in world space
float4 g_LightAmbient = { 1.0f, 1.0f, 1.0f, 1.0f };			// Light's ambient color
float4 g_LightDiffuse = { 1.0f, 1.0f, 1.0f, 1.0f };			// Light's diffuse color

texture Texture0;			// Color texture for mesh

float4x4 WorldViewProjection;	// World * View * Projection matrix
static const int MAX_MATRICES = 20;
float4x3    BoneMat[MAX_MATRICES] : WORLDMATRIXARRAY;
float4x4    ViewProjection : VIEWPROJECTION;
//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    Texture = <Texture0>;    
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


float3 Diffuse(float3 Normal)
{
    float CosTheta;
    
    // N.L Clamped
    CosTheta = max(0.0f, dot(Normal, lhtDir.xyz));
       
    // propogate scalar result to vector
    return (CosTheta);
}
//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   : POSITION;   // vertex position 
	float4 Color: COLOR0;
	float2 TextureUV : TEXCOORD0;
};


//-----------------------------------------------------------------------------
// Name: RenderSceneVS
// Type: Vertex shader                                      
// Desc: This shader computes standard transform and lighting
//-----------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
						float3 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0,                         
						float4  BlendWeights    : BLENDWEIGHT,
						float4  BlendIndices    : BLENDINDICES
						)
{
    VS_OUTPUT Output;
    // Transform the position from obj	ect space to homogeneous projection space
	// Animation the vertex based on time and the vertex's object space position
   // float4 vAnimatedPos = vPos;//mul(vPos,WorldMat);// + float4( sin(Time) * vPos.x/2, 0, 0, 0 );
    float3      Pos = 0.0f;
    float3      Normal = 0.0f;    
    float       LastWeight = 0.0f;
    int NumBones = 2;
    // Compensate for lack of UBYTE4 on Geforce3
    int4 IndexVector = D3DCOLORtoUBYTE4(BlendIndices);

    // cast the vectors to arrays for use in the for loop below
    float BlendWeightsArray[4] = (float[4])BlendWeights;
    int   IndexArray[4]        = (int[4])IndexVector;
    
    // calculate the pos/normal using the "normal" weights 
    //        and accumulate the weights to calculate the last weight
    for (int iBone = 0; iBone < NumBones-1; iBone++)
    {
        LastWeight = LastWeight + BlendWeightsArray[iBone];
        
        Pos += mul(vPos, BoneMat[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        Normal += mul(vNormal, BoneMat[IndexArray[iBone]]) * BlendWeightsArray[iBone];
    }
    LastWeight = 1.0f - LastWeight; 

    // Now that we have the calculated weight, add in the final influence
    Pos += (mul(vPos, BoneMat[IndexArray[NumBones-1]]) * LastWeight);
    Normal += (mul(vNormal, BoneMat[IndexArray[NumBones-1]]) * LastWeight); 
    
	// normalize normals
    Normal = normalize(Normal);
    
    // transform position from world space into view and then projection space
    Output.Position = mul(float4(Pos.xyz, 1.0f), ViewProjection);

    // Transform the position from object space to homogeneous projection space
    //Output.Position = mul(vPos, WorldViewProjection);
    Output.TextureUV = vTexCoord0;
       
	Output.Color = float4( Diffuse(Normal), 1);
    return Output;    
}


//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


//-----------------------------------------------------------------------------
// Name: RenderScenePS                                        
// Type: Pixel shader
// Desc: This shader outputs the pixel's color by modulating the texture's
//		 color with diffuse material color
//-----------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT In ) 
{ 
    PS_OUTPUT Output;

    // Lookup mesh texture and modulate it with diffuse
    //Output.RGBColor = float4(1,.2,.1,1);//tex2D(MeshTextureSampler, In.TextureUV);//*
	Output.RGBColor =In.Color;
    return Output;
}



//-----------------------------------------------------------------------------
// Name: RenderScene
// Type: Technique                                     
// Desc: Renders scene to render target
//-----------------------------------------------------------------------------
technique DefaultRender
{
    pass P0
    {          
        VertexShader = compile vs_1_1 RenderSceneVS();
        //PixelShader  = compile ps_1_1 RenderScenePS(); // trival pixel shader (could use FF instead if desired)
    }
}
